// Package Tools
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1330 $ on $Date: 2010-06-23 12:32:28 -0700 (Wed, 23 Jun 2010) $

// pragmas

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=1.0
#pragma hide=1

// package parameters

Static StrConstant thePackage="PackageTools"
Static StrConstant thePackageFolder="root:Packages:PackageTools"
Static StrConstant theProcedureFile = "PackageTools.ipf"
Static StrConstant thePackageInfo = "Tools to manage packages"
Static StrConstant thePackageAuthor = "Jeffrey J Weimer"
Static Constant thePackageVersion = 1.0
Static Constant hasHelp = 0

// primary functions

// hook to set up this package when an experiment starts

Static Function IgorStartorNewHook(igorApplicationNameStr)
	String igorApplicationNameStr

	SVAR pt = root:Packages:PackageTools:PackageTools
	
	if (!SVAR_Exists(pt))
		// intitalize package information
		DFREF cdf = GetDataFolderDFR()
		NewDataFolder/O/S Packages
		NewDataFolder/S PackageTools
		SetDataFolder cdf
		PackageSetup(thePackage,file=theProcedureFile,info=thePackageInfo,author=thePackageAuthor,version=thePackageVersion,hasHelp=hasHelp)
	else
		// update package information
		PackageSetup(thePackage,version=thePackageVersion)
	endif
	return 0
end

// function to set up package parameters

Function PackageSetup(name,[folder,version,info,author,file,hasHelp])
	string name
	string folder,info,author,file
	variable version, hasHelp
	
	// store current folder
	
	DFREF cdf = GetDataFolderDFR()
	
	// local parameters
	
	string thePackage, theFolder
	
	// set the package name and folder
	
	thePackage = CleanUpName(name,0)
	if (ParamIsDefault(folder))
		theFolder = thePackage
	else
		theFolder = CleanUpName(folder,0)
	endif

	// initialize the package parameter
	
	SetDataFolder $thePackageFolder
	SVAR ptStr = $thePackage		
	if (!SVAR_exists(ptStr))
		string/G $thePackage
		SVAR ptStr = $thePackage
		sprintf ptStr, "package:%s;folder:%s;", thePackage, theFolder
	endif
	
	// store the remaining package parameter values
	
	if (!ParamIsDefault(file))
		if (strsearch(file,":",0)>0)
			file = ParseFilePath(0,file,":",1,0)
		endif
		ptStr=ReplaceStringByKey("file",ptStr,file)
	endif
	
	if (!ParamIsDefault(info))
		ptStr=ReplaceStringByKey("info",ptStr,info)
	endif
	
	if (!ParamIsDefault(author))
		ptStr=ReplaceStringByKey("author",ptStr,author)
	endif
	
	if (!ParamIsDefault(version))
		ptStr=ReplaceNumberByKey("version",ptStr,version)
	endif
	
	if (!ParamIsDefault(hasHelp))
		ptStr=ReplaceNumberByKey("hasHelp",ptStr,hasHelp)
	endif
	
	// done
	
	SetDataFolder cdf
	
	return 0
	
end

// function to display help for package

Function PackageDisplayHelp(name)
	string name

	string hasHelp = PackageKey(name,"hasHelp")

	// check for non-registered or non-existent help
	
	if ( strlen(hasHelp)==0 || !cmpstr(hasHelp,"0") )
		DoAlert 0, "NO HELP REGISTERED FOR PACKAGE: " + name
		return 0
	endif

	// show help
	
	DisplayHelpTopic name
	return 0

end

// function to remove package

Function PackageRemove(name,[quiet])
	string name
	variable quiet
	
	// confirm request if not quiet
	
	if (ParamIsDefault(quiet) || (quiet == 0))
		DoAlert 1, "Do you really want to remove the package " + name + "?"	
		if (V_flag == 2)
			return 0
		endif
	endif
	
	string theCmd
	
	// kill the package folder
	
	theCmd = PackageKey(name,"folder")
	
	if (strlen(theCmd)!=0)
		sprintf theCmd, "root:Packages:%s", theCmd
		KillDataFolder/Z $theCmd
	endif
	
	// delete the package include statement
	
	sprintf theCmd "DELETEINCLUDE \"%s\"" name
	Execute/P/Z/Q theCmd

	// close the package procedure file
	
	theCmd = PackageKey(name,"file")
	if (strlen(theCmd)!=0)
		sprintf theCmd "CloseProc/NAME=\"%s\"" theCmd	
		Execute/P/Z/Q theCmd
	endif
	
	return 0
	
end

// support functions

// get a key value for the package

Function/S PackageKey(name,key)
	string name, key
	
	DFREF ptdf = root:Packages:PackageTools
	
	// return blank if package tools not installed
	
	if (DataFolderRefStatus(ptdf) == 0)
		return ""
	endif
	
	// get and return the package key string
	
	SVAR/SDFR=ptdf ptStr=$name
	
	if (SVAR_exists(ptStr))
		return (StringByKey(key,ptStr))
	else
		return ""
	endif
end

// get the DFREF location for a folder in a given package

Function/DF PackageFolderDFR(name,[folder])
	string name
	string folder
	
	string pdf = PackageKey(name,"folder")
	
	// get and return the package folder DFREF
	
	if (strlen(pdf)!=0)
		if (ParamIsDefault(folder))
			sprintf pdf, "root:Packages:%s", pdf
		else
			sprintf pdf, "root:Packages:%s:%s", pdf, folder
		endif
	endif
	
	DFREF pdfR = $pdf
	
	return pdfR
end
